#include "csvmanager.h"
#include <QDebug>
CSVManager::CSVManager(QObject *parent) : QObject(parent)
{
    connect(this,SIGNAL(convertionError(QString)),this,SLOT(showErrors(QString)));
    connect(this,SIGNAL(convertionWarning(QString)),this,SLOT(showErrors(QString)));
}
QList<QString> CSVManager::getListFromRow(QString oneRow){
    QList<QString> s = oneRow.split(";");
    //  qDebug() << s;
    return s;
}
void CSVManager::showErrors(QString s){
    qDebug() << s;
}
bool CSVManager::openCSVFileAt(QString s,int langId){
    if(QFile::exists(s) == false){
        return false;
    }
    QFile f;
    f.setFileName(s);
    if(!f.open(QIODevice::ReadWrite)){
        emit convertionError("File is still in use?!");
        return false;
    }
    QByteArray b = f.readAll();
    QString st = QString(QLatin1String(b));
    QList<QString> row = st.split("\r\n");
    QList<CSVRow> r;
    for(int i = 0; i < row.count();i++){
        CSVRow c;
        c.collumn = getListFromRow(row.at(i));
        r.append(c);
    }
    rows = r;
    if(row.count() <= 1){
        emit convertionError("Is the File Empty?");
        return false;
    }
    //gainedNewData(extractFromCSVOneLangMode(langId));
    QList<int> d;
    d.append(langId);

    UserDataBase usb = extractMultiLangMode(d);
    updateLangOfCategoryTo(&usb.categorys,langId);
    gainedNewData(usb);
}

bool CSVManager::openCSVFileAt(QString s, QList<int> lang)
{
    if(QFile::exists(s) == false){
        return false;
    }
    QFile f;
    f.setFileName(s);
    if(!f.open(QIODevice::ReadWrite)){
        emit convertionError("File is still in use?!");
        return false;
    }
    QByteArray b = f.readAll();
    QString st = QString(QLatin1String(b));
    QList<QString> row = st.split("\r\n");
    QList<CSVRow> r;
    for(int i = 0; i < row.count();i++){
        CSVRow c;
        c.collumn = getListFromRow(row.at(i));
        r.append(c);
    }
    rows = r;
    if(row.count() <= 1){
        emit convertionError("Is the File Empty?");
        return false;
    }
    //gainedNewData(extractFromCSVOneLangMode(langId));
    gainedNewData(extractMultiLangMode(lang));
}
bool CSVManager::openAlternativeCSVFileAt(QString s, QList<int> lang)
{
    if(QFile::exists(s) == false){
        return false;
    }
    QFile f;
    f.setFileName(s);
    if(!f.open(QIODevice::ReadWrite)){
        emit convertionError("File is still in use?!");
        return false;
    }
    QByteArray b = f.readAll();
    QString st = QString(QLatin1String(b));
    QList<QString> row = st.split("\r\n");
    QList<CSVRow> r;
    for(int i = 0; i < row.count();i++){
        CSVRow c;
        c.collumn = getListFromRow(row.at(i));
        r.append(c);
    }
    rows = r;
//    qDebug() << "IMPORT!!"<<lang.count();
    if(row.count() < 6){
        emit convertionError("Is the File Empty?");
        return false;
    }
    gainedNewData(extractFromCSVAltertiveCSVDataType(lang));
}
int CSVManager::langContains(QString kuerzel,QList<languageType>* ll){
    for(int i = 0; i < ll->count();i++){
        if(ll->at(i).lang == kuerzel)
            return i;
    }
    return -1;
}

bool CSVManager::isEmpty(QList<QString> s){
    for(int i = 0; i < s.count(); i++){
        if(!s.at(i).isEmpty())return false;
    }
    return true;
}
//multilanguage Support
UserDataBase CSVManager::extractMultiLangMode(QList<int> lang){
    int collumsPerLang = 12;
    int langCount = rows.at(1).collumn.count() / collumsPerLang;
    if(langCount > lang.count())
        langCount = lang.count();
    UserDataBase dd;
    if(lang.count() == 0)lang.append(-1);
    //one lang!
    UserDataBase d = extractFromCSVOneLangMode(lang.at(0),0*collumsPerLang,true);
    updateLangOfCategoryTo(&d.categorys,lang.at(0));
    //if more langs!
    for(int i = 1; i < langCount && i < lang.count();i++){
        int langID = lang.at(i);
        //  qDebug() << "GET Lang!";
        UserDataBase udb = extractFromCSVOneLangMode(langID,i*collumsPerLang,true);
        //qDebug() << "GOT Lang! & updateLangOfCategoryTo";

        // qDebug() << "updateLangOfCategoryTo DONE!";
        d.languages << udb.languages;


        for(int c = 0; c < udb.categorys.count();c++){
            if(d.categorys.count() > c){

                Lang_Text l = udb.categorys.at(c).text.at(0);
                Category* cc = (Category*)&d.categorys.at(c);
                cc->text.append(l);
            }
            else
                d.categorys.append(udb.categorys.at(c));

            for(int q = 0; q < udb.categorys.at(c).questions.count();q++){
                if(d.categorys.at(c).questions.count() > q){
                    Question* qq = (Question*)&d.categorys.at(c).questions.at(q);
                    qq->text.append(udb.categorys.at(c).questions.at(q).text.at(0));

                }
                else
                    ((Category*)&d.categorys.at(c))->addOneQuestion(udb.categorys.at(c).questions.at(q));

                for(int a = 0; a < udb.categorys.at(c).questions.at(q).answers.count();a++){

                    Answer* aNew;
                    Answer* aOld;
                    aNew = (Answer*)&udb.categorys.at(c).questions.at(q).answers.at(a);

                    if(d.categorys.at(c).questions.at(q).answers.count() > q){
                        aOld = (Answer*)&d.categorys.at(c).questions.at(q).answers.at(a);
                        aOld->text.append(aNew->text.at(0));
                        aOld->explanation.append(aNew->explanation.at(0));
                    }
                    else
                        ((Question*)&d.categorys.at(c).questions.at(q))->addOneAnswer(*aNew);
                }
            }
        }
        updateLangOfCategoryTo(&udb.categorys,lang.at(i));

    }

    dd.categorys = d.categorys;
    if(lang.at(0) == -1)
        dd.languages = d.languages;
    return dd;
}
UserDataBase CSVManager::extractFromCSVAltertiveCSVDataType(QList<int> langIds){
    QList<languageType> ll;
    QList<Category> cs;

    messageBox.hide();
    //3 langs are NEEDED!
    if(langIds.count() < 3){
        messageBox.critical(0,"Import Error!"," You only use "+QString::number(langIds.count())+
                            " but you need atleast 3 Languages \n "
                            "to import Data from this CSV Type! \n ");
        messageBox.setFixedSize(500,200);
       messageBox.show();return UserDataBase();
    }
    //    if(langIds.length() == 0)langIds.append(1);//<<=alternative solution to go around that language limit, but it could be dangerous for the database
    //    if(langIds.length() == 1)langIds.append(2);
    //    if(langIds.length() == 2)langIds.append(3);
    //--------

    for(int r = 0; r < rows.count()&& (rows.count()/6)*6 > r;r++){

        QList<QString>* col = (QList<QString>*)&rows.at(r).collumn;
        if(col->count() < 6){
            for(int i = col->count()-1; i < 8;i++){
                col->append(" SAMPLE TEXT ");
//                qDebug() << "space!";
            }

        }

        int mode = r % 6;// 6 rows per category with all things

        if(mode == 0){
            Category c = Category(-1);
            c.text << Lang_Text(langIds[0],col->at(0),ADD_);
            c.text << Lang_Text(langIds[1],col->at(1),ADD_);
            c.text << Lang_Text(langIds[2],col->at(2),ADD_);
            cs.append(c);
        }
        if(mode == 1){
            Question q = Question(-1);
            q.text << Lang_Text(langIds[0],col->at(0),ADD_);
            q.text << Lang_Text(langIds[1],col->at(1),ADD_);
            q.text << Lang_Text(langIds[2],col->at(2),ADD_);
            cs.last().addOneQuestion(q);
        }
        if(mode > 1){


            Answer a = Answer(-1,(Question*)&cs.last().questions.last());

            a.text << Lang_Text(langIds[0],col->at(0),ADD_);
            a.text << Lang_Text(langIds[1],col->at(1),ADD_);
            a.text << Lang_Text(langIds[2],col->at(2),ADD_);

            if(mode == 2){
                a.setCorrect(true);
                a.explanation << Lang_Text(langIds[0],col->at(3),ADD_);
                a.explanation << Lang_Text(langIds[1],col->at(4),ADD_);

                a.explanation << Lang_Text(langIds[2],col->at(5),ADD_);
            }
            else
                a.setCorrect(false);
            cs.last().questions.last().addOneAnswer(a);

        }
    }
    qDebug() << "hans2";
    setAllToAddMode(&cs);
    UserDataBase usb;
    usb.languages = ll;
    usb.categorys = cs;
    return usb;
}
UserDataBase CSVManager::extractFromCSVOneLangMode(int lang_id, int offset, bool multilang){
    //index 0 is THE DESCRIPTION
    QList<languageType> ll;
    QList<Category> cs;

    //qDebug() << "rows "<<rows.count();
    for(int r = 1; r < rows.count();r++){
        QList<QString>* col = (QList<QString>*)&rows.at(r).collumn;
        if(col->count() < offset+11){
            if(!multilang){
                emit this->convertionWarning("Missing Collumns!");continue;
            }
            else{
                for(int i = col->count(); i < offset+14;i++){
                    col->append("SAMPLE TEXT");
                }
                continue;
            }
        }
        if(!multilang)
            if(isEmpty(*col))continue;

        int langId = langContains(col->at(offset+0),&ll);
        if(langId == -1){
            ll.append(languageType(ll.count(),col->at(offset+1),col->at(offset+0)));
            langId = ll.count()-1;
        }
        int trick17 = offset+1;
        Category c = Category(-1);
        c.text << Lang_Text(langId,col->at(trick17++),ADD_);
        Question q = Question(-1,&c);
        q.text << Lang_Text(langId,col->at(trick17++),ADD_);



        Answer right=Answer(0,&q);right.text << Lang_Text(-1,col->at(trick17++),ADD_);
        right.explanation << Lang_Text(-1,col->at(trick17++),ADD_);
        right.setCorrect(true);
        Answer wrong=Answer(0,&q),wrong2=Answer(0,&q),wrong3=Answer(0,&q);
        wrong.text  << Lang_Text(langId,col->at(trick17++),ADD_) ;
        wrong.explanation << Lang_Text(langId,col->at(trick17++),ADD_);
        wrong.setCorrect(false);
        wrong2.text << Lang_Text(langId,col->at(trick17++),ADD_);
        wrong2.explanation << Lang_Text(langId,col->at(trick17++),ADD_);
        wrong2.setCorrect(false);
        wrong3.text << Lang_Text(langId,col->at(trick17++),ADD_);
        wrong3.explanation << Lang_Text(langId,col->at(trick17++),ADD_);
        wrong3.setCorrect(false);

        q.addOneAnswer(right) ;q.addOneAnswer(wrong);
        q.addOneAnswer(wrong2);q.addOneAnswer(wrong3);
        c.questions.append(q);
        cs.append(c);
    }

    for(int c = 0; c < cs.length();c++){
        QList<Question> q;
        for(int x = 0; x < cs.length();x++){
            if(x == c)continue;
            if(cs.at(c).text.at(0).text == cs.at(x).text.at(0).text)
            {
                q<<cs.at(x).questions;
                cs.removeAt(x);
                x--;
            }
        }
        *((QList<Question>*)&cs.at(c).questions) << q;
    }




    UserDataBase usb;
    usb.languages = ll;
    usb.categorys = cs;
    updateLangOfCategoryTo(&usb.categorys,lang_id);
    usb.printAll();

    return usb;
}
QList<Category> CSVManager::setAllJobsTo(QList<Category> cs,int newLangId){
    for(int c = 0; c < cs.length();c++){
        Category* cat = (Category*)&cs.at(c);

        if(newLangId!=-1){

            if(cat->text.count() > 0)((Lang_Text*)&cat->text.at(0))->lang_id = newLangId;
        }

        cat->job = ADD_;
        for(int q = 0; q < cs.at(c).questions.count();q++){
            Question* qq = (Question*)&cs.at(c).questions.at(q);
            qq->job = ADD_;
            if(newLangId!=-1){
                if(qq->text.count() > 0)((Lang_Text*)&qq->text.at(0))->lang_id = newLangId;

            }

            for(int a = 0; a < qq->answers.count();a++){

                Answer* aa = (Answer*)&qq->answers.at(a);

                aa->job = ADD_;
                if(newLangId!=-1){
                    if(aa->text.count() > 0)((Lang_Text*)&aa->text.at(0))->lang_id = newLangId;
                }
            }
        }
    }
    return cs;
}
