#include "editwidget.h"
#include <QDebug>
EditWidget::EditWidget(CurrentEditType cET, QWidget *parent) : QWidget(parent)
{
    parentEditWidget = Q_NULLPTR;
    curLang = 0;
    iAmBlue = false;
    listView = new QListWidget();
    hinzufuegenBtn   = new QPushButton("Add");
    deleteBtn        = new QPushButton("Delete");
    editBtn          = new QPushButton("Edit");
    continueBtn      = new QPushButton("Select");
    infoLabel = new QLabel("Description");
    infoLabel->setAlignment(Qt::AlignHCenter);
    QGridLayout* layout = new QGridLayout();
    layout->addWidget(infoLabel   ,0,0,1,4);
    layout->addWidget(listView   ,1,0,10,4);

    layout->addWidget(continueBtn    ,1,5);
    layout->addWidget(hinzufuegenBtn ,2,5);
    layout->addWidget(editBtn        ,3,5);
    layout->addWidget(deleteBtn      ,4,5);
    this->setLayout  (layout);

    setMode(cET);

    cList = Q_NULLPTR;
    qList = Q_NULLPTR;
    aList = Q_NULLPTR;
    connect(continueBtn     ,SIGNAL(clicked(bool)),this,SLOT(onSelectAndContinue()));
    connect(deleteBtn       ,SIGNAL(clicked(bool)),this,SLOT(selectionDelete()));
    connect(editBtn         ,SIGNAL(clicked(bool)),this,SLOT(selectionEdit()));
    connect(hinzufuegenBtn  ,SIGNAL(clicked(bool)),this,SLOT(addAndEditNewElement()));
    connect(listView,SIGNAL(itemSelectionChanged()),this,SLOT(renameDeleteBtn()));
    connect(listView,SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(onSelectAndContinue()));
    connect(listView,SIGNAL(itemDoubleClicked(QListWidgetItem*)),this,SLOT(selectionEdit()));
    this->setEnabled(false);
    updateText();
}
void EditWidget::addAndEditNewElement(){
    emit addTypeToList(mode);
}
bool EditWidget::getIsBlue(){
    return iAmBlue;
}
QList<Category>* EditWidget::getCList(){ return cList;}
QList<Question>* EditWidget::getQList(){ return qList;}
QList<Answer  >* EditWidget::getAList(){ return aList;}
void EditWidget::setMode(CurrentEditType t){
    mode = t;//do smthg
    if(mode == ANSWER)
        continueBtn->setVisible(false);
    else
        continueBtn->setVisible(true);
}
void EditWidget::turnOffSelectAble(){
    this->cleanAllData();
    this->setEnabled(false);
}
void EditWidget::onSelectAndContinue(){
    if(mode == NOTHING_ || mode == ANSWER)return;
    if(listView->selectedItems().count() > 0)
    {
        int r = listView->row(listView->selectedItems().at(0));
        if(cList != Q_NULLPTR && mode == CATEGORY)
            if(r < cList->count())
                emit this->SelectedData((QList<Question>*)&cList->at(r).questions,(Category*)&cList->at(r));
        if(qList != Q_NULLPTR && mode == QUESTION)
            if(r < qList->count())
                emit this->SelectedData((QList<Answer>*)&qList->at(r).answers,(Question*)&qList->at(r));
    }
}
void EditWidget::openAllData(QList<Category>* c){
    if(cList == c)return;
    cList = c;
    mode = CATEGORY;
    //qDebug() << "sync";
    resyncList();
    this->setEnabled(true);
    emit onNewData();
}
void EditWidget::openAllData(QList<Question>* q,Category* owner){
    if(qList == q)return;
    ownerOfQList = owner;
    qList = q;
    mode = QUESTION;
    resyncList();
    this->setEnabled(true);
    emit onNewData();
}
void EditWidget::cleanAllData(){
    cList = Q_NULLPTR;
    qList = Q_NULLPTR;
    aList = Q_NULLPTR;
    ownerOfAList = Q_NULLPTR;
    ownerOfQList = Q_NULLPTR;
    mode = NOTHING_;

    resyncList();
    this->setEnabled(false);

}
void EditWidget::openAllData(QList<Answer>* a,Question* owner){
    if(aList == a)return;
    ownerOfAList = owner;
    aList = a;
    mode = ANSWER;
    resyncList();
    this->setEnabled(true);
    emit onNewData();
}
Category* EditWidget::getOwnerOfQList(){
    return this->ownerOfQList;
}
Question*   EditWidget::getOwnerOfAList(){
    return this->ownerOfAList;
}
void EditWidget::selectionEdit(){
    if(mode == NOTHING_)return;
    if(listView->selectedItems().count() > 0)
    {
        int r = listView->row(listView->selectedItems().at(0));
        if(cList != Q_NULLPTR && mode == CATEGORY)
            if(r < cList->count())
                emit SelectedToEdit((Category*)&cList->at(r));
        if(qList != Q_NULLPTR && mode == QUESTION)
            if(r < qList->count())
                emit SelectedToEdit(((Question*)&qList->at(r)));
        if(aList != Q_NULLPTR && mode == ANSWER)
            if(r < aList->count()){
                emit SelectedToEdit((Answer   *)&aList->at(r));
//                qDebug() << " YELLOW ";
            }
    }
}
void EditWidget::setParentEditWidget(EditWidget* nParent){
    parentEditWidget = nParent;
}
void EditWidget::selectionDelete(){

    if(mode == NOTHING_)return;

    if(listView->selectedItems().count() > 0)
    {
        int r = listView->row(listView->selectedItems().at(0));
        if(cList != Q_NULLPTR && mode == CATEGORY){
            if(r < cList->count()){
                if(cList->at(r).job == DELETE_)
                    ((Category*)&cList->at(r))->job = EDIT;
                else if(cList->at(r).job != ADD_)
                    ((Category*)&cList->at(r))->job = DELETE_;
                else
                {
                    cList->removeAt(r);
                    emit instantDeletedStuff();
                }
            }
        }
        if(qList != Q_NULLPTR && mode == QUESTION){
            if(r < qList->count()){
                if(qList->at(r).job == DELETE_)
                    ((Question*)&qList->at(r))->job = EDIT;
                else if(qList->at(r).job != ADD_)
                    ((Question*)&qList->at(r))->job = DELETE_;
                else
                {
                    qList->removeAt(r);
                    emit instantDeletedStuff();
                }
            }
        }
        if(aList != Q_NULLPTR && mode == ANSWER){
            if(r < aList->count()){
                if(aList->at(r).job == DELETE_)
                    ((Answer  *)&aList->at(r))->job = EDIT;
                else if(aList->at(r).job != ADD_)
                    ((Answer  *)&aList->at(r))->job = DELETE_;
                else
                {
                    aList->removeAt(r);
                    emit instantDeletedStuff();
                }
            }
        }
    }
    renameDeleteBtn();
    resyncList();
    emit onDeletingStuff();
}
void EditWidget::renameDeleteBtn(){
    bool isDeleted = false;
    if(listView->selectedItems().count() > 0)
    {
        int r = listView->row(listView->selectedItems().at(0));
        if(cList != Q_NULLPTR && mode == CATEGORY)
            if(r < cList->count())
                if(cList->at(r).job == DELETE_)
                    isDeleted = true;
        if(qList != Q_NULLPTR && mode == QUESTION)
            if(r < qList->count())
                if(qList->at(r).job == DELETE_)
                    isDeleted = true;
        if(aList != Q_NULLPTR && mode == ANSWER)
            if(r < aList->count()){
                if(aList->at(r).job == DELETE_)
                    isDeleted = true;
            }
    }
    if(isDeleted)
        deleteBtn->setText(tr("Dont Delete"));
    else
        deleteBtn->setText(tr("Delete"));
}
void EditWidget::resyncList(){
    //current Language
    int langId = curLang;//GET CURRENT LANGUAGE

    listView->clear();
    if(mode == NOTHING_)return;

    if(mode == CATEGORY && cList != Q_NULLPTR){
        for(int i = 0; i < cList->count();i++){
            qDebug() << i;
            QString s = "";
            if(langId < cList->at(i).text.count())
                s = cList->at(i).text.at(langId).text;

            QListWidgetItem* qlwi = new QListWidgetItem();

            if(cList->at(i).job == ADD_)
                qlwi->setData(1,QColor(Qt::cyan));
            else if( cList->at(i).job == EDIT){
                qlwi->setData(1,QColor(Qt::yellow));
            }
            else if(cList->at(i).job == DELETE_){
                qlwi->setData(1,QColor(Qt::red));
            }
            else
                qlwi->setData(Qt::DecorationRole,QColor(Qt::green));

            qlwi->setText(s);
            listView->addItem(qlwi);
        }
    }
    else if(mode == QUESTION && qList != Q_NULLPTR){
        iAmBlue = false;
        for(int i = 0; i < qList->count();i++){
            QString s = "";
            if(langId < qList->at(i).text.count())
                s = qList->at(i).text.at(langId).text;

            QListWidgetItem* qlwi = new QListWidgetItem();
            bool okay = true;
            if(ownerOfQList!=Q_NULLPTR)
                if(ownerOfQList->job == DELETE_){
                    okay = false;
                }
            if(!okay){
                qlwi->setData(1,QColor(Qt::black));
                iAmBlue = true;
            }
            else if(qList->at(i).job == ADD_)
                qlwi->setData(1,QColor(Qt::cyan));

            else if( qList->at(i).job == EDIT){
                qlwi->setData(1,QColor(Qt::yellow));
            }
            else if(qList->at(i).job == DELETE_){
                qlwi->setData(1,QColor(Qt::red));
            }
            else
                qlwi->setData(1,QColor(Qt::green));

            qlwi->setText(s);
            listView->addItem(qlwi);
        }
    }
    else if(mode == ANSWER && aList != Q_NULLPTR){
        iAmBlue = false;
        for(int i = 0; i < aList->count();i++){

            QString s = "";
            if(langId < aList->at(i).text.count())
                s = aList->at(i).text.at(langId).text;
            QListWidgetItem* qlwi = new QListWidgetItem();
            bool okay = true;
            if(ownerOfAList!=Q_NULLPTR)
                if(ownerOfAList->job == DELETE_){
                    okay = false;
                }
            if(parentEditWidget != Q_NULLPTR)
                if(parentEditWidget->getIsBlue())
                    okay = false;
            if(!okay){
                qlwi->setData(1,QColor(Qt::black));
                iAmBlue = true;
            }
            else if(aList->at(i).job == ADD_)
                qlwi->setData(1,QColor(Qt::cyan));
            else if (aList->at(i).job == EDIT){
                qlwi->setData(1,QColor(Qt::yellow));
            }
            else if(aList->at(i).job == DELETE_){
                qlwi->setData(1,QColor(Qt::red));
            }
            else
                qlwi->setData(1,QColor(Qt::green));
            if(aList->at(i).isCorrect())
                qlwi->setTextColor(QColor(0,80,0));
                //s.append("\t isCorrect = true");

            qlwi->setText(s);

            listView->addItem(qlwi);
        }
    }
}
void EditWidget::setStdLang(int d){
    curLang = d;
    resyncList();
    //qDebug() << "  "<<d;
}
void EditWidget::updateText(){
    hinzufuegenBtn->setText("Add");
    editBtn->setText("Edit");
    continueBtn->setText(tr("Select"));
    renameDeleteBtn();
    if(mode == ANSWER)infoLabel->setText(tr("Answers"));
    else if(mode == CATEGORY) infoLabel->setText(tr("Categories"));
    else if(mode == QUESTION) infoLabel->setText(tr("Questions"));
}
