#include "popupeditor.h"

PopUpEditor::PopUpEditor(QWidget *parent) : QWidget(parent)
{
    currentJob = CATEGORY;
    QGridLayout* layout = new QGridLayout;

    accept = new QPushButton();
    abort  = new QPushButton();

    textOneEdit = new QLineEdit();
    textTwoEdit = new QLineEdit();
    textThreeEdit = new QLineEdit();
    sprache = new QLabel("Language:");
    dTxt1   = new QLabel(" Question: ");
    dTxt2   = new QLabel(" Hint:");
    dTxt3   = new QLabel(" Explanation:");

    langChooser = new QComboBox();
    isCorrect   = new QCheckBox("Is correct!");


    layout->addWidget(sprache,0,0);
    layout->addWidget(langChooser,0,1);

    layout->addWidget(dTxt1,1,0);
    layout->addWidget(textOneEdit,1,1,1,3);
    layout->addWidget(dTxt2,2,0);
    layout->addWidget(textTwoEdit,2,1,1,3);
    layout->addWidget(dTxt3,3,0);
    layout->addWidget(textThreeEdit,3,1,1,3);
    layout->addWidget(isCorrect,4,1,1,3);

    layout->addWidget(accept,6,2);

    layout->addWidget(abort ,6,3);

    layout->setColumnStretch(1,10);
    this->setLayout(layout);
    updateText();

    connect(textOneEdit,SIGNAL(editingFinished()) ,this ,SLOT(text1EditFinished()));
    connect(textTwoEdit,SIGNAL(editingFinished()) ,this ,SLOT(text2EditFinished()));
    connect(textThreeEdit,SIGNAL(editingFinished()) ,this ,SLOT(text3EditFinished()));
    connect(isCorrect  ,SIGNAL(clicked(bool))     ,this ,SLOT(CheckBoxEditFinished()));
    connect(langChooser,SIGNAL(currentIndexChanged(int)),this,SLOT(langIndexChanged()));
    connect(accept     ,SIGNAL(clicked(bool))     ,this,SLOT(onEditFinished()));
    connect(abort      ,SIGNAL(clicked(bool))     ,this,SLOT(onEditAborted()));
}
void PopUpEditor::editCategory(Category* c){
    if(c == Q_NULLPTR)return;
    desableAll();
    this->cat = c;
    //dTxt2->setVisible(true);
   // textTwoEdit->setVisible(true);
    currentJob = CATEGORY;

    text_One = c->text;
    text_One = reOrderTextByLangId(&text_One);
    //text_Two = reOrderTextByLangId(&text_Two);
//    qDebug() << " W1 ";
    updateText();
    langIndexChanged();
    this->show();this->setFocus();
}
void PopUpEditor::editQuestion(Question* q){
    if(q == Q_NULLPTR)return;
    desableAll();

    this->q = q;

    //dTxt2->setVisible(true);
    //dTxt3->setVisible(true);
    //textTwoEdit->setVisible(true);
   // textThreeEdit->setVisible(true);
    currentJob = QUESTION;
    text_One = q->text;


    text_One = reOrderTextByLangId(&text_One);

    updateText();
    langIndexChanged();
    this->show();this->setFocus();
}

void PopUpEditor::editAnswer  (Answer*   a){
    qDebug() << a;
    if(a == Q_NULLPTR)return;
    desableAll();
    this->a = a;
    isCorrect->setVisible(true);
    dTxt2->setVisible(true);
    textTwoEdit->setVisible(true);
    currentJob = ANSWER;
    text_One = a->text;
    text_Two = a->explanation;
//    for(int l = 0; l < text_One.count();l++){
//        qDebug() << "txt 1  " << text_One.at(l).text;
//        qDebug() << "lang id" << text_One.at(l).lang_id;
//    }
    text_One = reOrderTextByLangId(&text_One);
//    qDebug() << "pt2";
//    for(int l = 0; l < text_Two.count();l++){
//        qDebug() << "txt 2  " << text_Two.at(l).text;
//        qDebug() << "lang id" << text_Two.at(l).lang_id;
//    }
    text_Two = reOrderTextByLangId(&text_Two);
    updateText();
    langIndexChanged();
    isCorrect->setChecked(a->isCorrect());
    this->show();this->setFocus();
}
QList<Lang_Text> PopUpEditor::reOrderTextByLangId(QList<Lang_Text>* ltl){
    QList<Lang_Text> txt1;
    for(int i = 0; i < languages.count();i++){
        if(languages.at(i).job == ADD_)continue;
        int id = languages.at(i).getLangId();
        int finalId = -1;
        for(int t = 0; t < ltl->count();t++){
            if(ltl->at(t).lang_id == id){
                finalId = t;
                break;
            }
        }
        Lang_Text nlt;
        if(finalId == -1){
            nlt = Lang_Text(id," SAMPLE TEXT ");
            nlt.job = ADD_;
        }
        else
            nlt = ltl->at(finalId);

        txt1.append(nlt);
    }
    return txt1;
}
void PopUpEditor::desableAll(){
    textTwoEdit->setVisible(false);
    textThreeEdit->setVisible(false);
    dTxt2->setVisible(false);
    dTxt3->setVisible(false);
    isCorrect->setVisible(false);
}
void PopUpEditor::onEditFinished(){
    desableAll();
    if(currentJob == CATEGORY)
    {

        cat->text = this->text_One;
        if(cat->job != ADD_)
            cat->job  = EDIT;
    }
    else if(currentJob == QUESTION){
        q->text =  this->text_One;


        if(q->job != ADD_)
            q->job  = EDIT;
    }
    else if(currentJob == ANSWER){
        a->text = this->text_One;
        a->explanation = this->text_Two;
        a->setCorrect(isCorrect->isChecked());
        if(a->job != ADD_)
            a->job  = EDIT;
    }
    this->close();
    emit editFinished();
    cleanAllData();
}
void PopUpEditor::onEditAborted(){
    this->close();
    cleanAllData();
    emit editAborted();
}
void PopUpEditor::updateText(){
    accept->setText (tr("&Confirm"));
    abort->setText  (tr("&Abort"));
    sprache->setText(tr("Language:"));
    if(currentJob == CATEGORY){
        dTxt1->setText(tr("Text"));
        this->setWindowTitle(tr("Edit the Category"));
    }
    if(currentJob == QUESTION){
        dTxt1->setText(tr("Question:"));
        //dTxt2->setText(tr("Hinweis:"));
        //dTxt3->setText(tr("Erklärung:"));
        this->setWindowTitle(tr("Edit the Question"));
    }
    if(currentJob == ANSWER){
        dTxt1->setText(tr("Answer:"));
        dTxt2->setText(tr("Explanation:"));
        this->setWindowTitle(tr("Edit the Answer"));
    }

}
void PopUpEditor::updateLanguges(QList<languageType> lt){
    this->languages = lt;


    langChooser->blockSignals(true);
    langChooser->clear();


    for(int i = 0; i < lt.count();i++){
        langChooser->addItem(lt.at(i).lang);
    }
    langChooser->blockSignals(false);

}

void PopUpEditor::setCurLangId(int l)
{
    if(l > langChooser->count())return;
    langChooser->blockSignals(true);
    langChooser->setCurrentIndex(l);
    langChooser->blockSignals(false);
}
void PopUpEditor::text1EditFinished(){
    int index = langChooser->currentIndex();
    if(index >= text_One.length()){
        text_One = reOrderTextByLangId(&text_One);
    }
    if(index >= text_One.length())return; // dann weis ich auch nicht!
    if(currentJob != NOTHING_){
        ((Lang_Text*)&text_One.at(index))->text = this->textOneEdit->text();
    }
}
void PopUpEditor::cleanAllData(){
    cat = Q_NULLPTR;
    q = Q_NULLPTR;
    a = Q_NULLPTR;
    this->text_One = QList<Lang_Text>();
    this->text_Two = QList<Lang_Text>();
    this->text_Three = QList<Lang_Text>();
    //lang behalten wir JO
}
void PopUpEditor::text2EditFinished(){
    int index = langChooser->currentIndex();
    if(index >= text_Two.length()){
        text_Two = reOrderTextByLangId(&text_Two);
    }
    if(index >= text_Two.length())return; // dann weis ich auch nicht!
    if(currentJob != NOTHING_ ){
        ((Lang_Text*)&text_Two.at(index))->text = this->textTwoEdit->text();
    }
}
void PopUpEditor::text3EditFinished(){
    int index = langChooser->currentIndex();
    if(index >= text_Three.length()){
        text_Three = reOrderTextByLangId(&text_Three);
    }
    if(index >= text_Three.length())return; // dann weis ich auch nicht!
    if(currentJob != NOTHING_ && currentJob != ANSWER){
        ((Lang_Text*)&text_Three.at(index))->text = this->textThreeEdit->text();
    }
}
void PopUpEditor::langIndexChanged(){
    int newIndex = langChooser->currentIndex();
    if(text_One.length() > newIndex)
        this->textOneEdit->setText(text_One.at(newIndex).text);
    if(currentJob != NOTHING_)
        if(text_Two.length() > newIndex)
            this->textTwoEdit->setText(text_Two.at(newIndex).text);
    if(currentJob != ANSWER && currentJob != NOTHING_)
        if(text_Three.length() > newIndex)
            this->textThreeEdit->setText(text_Three.at(newIndex).text);
//    qDebug() << "C index "<< newIndex;
//    for(int l = 0; l < languages.count();l++){

//        qDebug() << "index "<< l;
//        qDebug() << "lang name = " << languages.at(l).lang;
//        qDebug() << "lang id   = " << languages.at(l).getLangId();
//    }
//    qDebug() << " check it ";
//    for(int l = 0; l < text_One.count();l++){
//        qDebug() << "txt 1  " << text_One.at(l).text;
//        qDebug() << "lang id" << text_One.at(l).lang_id;
//    }
}
void PopUpEditor::CheckBoxEditFinished(){
    if(currentJob == ANSWER){
        a->setCorrect(this->isCorrect->isChecked());
        if(a->job != ADD_)
            a->job = EDIT;
    }
}
